from bokeh.plotting import figure, output_file, show, ColumnDataSource
from bokeh.palettes import viridis
from bokeh.models.widgets import Slider
from bokeh.models import CustomJS, CustomJSFilter, CDSView, ColorBar
from bokeh.layouts import row, column
from bokeh.transform import linear_cmap
import sys


def bokeh_xy_sliders(outputMatrix):
    # outputArray is a 2d array composed of
    # a transposed array of unique group names, competitiveness scores,
    # comp p-values, lottery scores, and lottery p-values.
    # Scatterplot with sliders to filter significance
    output_file("scatterplot.html")

    TOOLS = ("hover,pan,wheel_zoom,zoom_in,zoom_out,box_zoom,reset, save, tap")

    TOOLTIPS = [  # Displays on hover
        ("group", "@groups"),
        ("Comp P-value", "@compP{0.0000}"),
        ("Lottery P-value", "@lotP{0.0000}"),
        ("(x,y)", "(@x, @y)")
    ]

    # create a new plot with a title and axis labels
    p = figure(tools=TOOLS, title="Scatterplot", x_axis_label='Lottery Score',
               y_axis_label='Competitiveness', tooltips=TOOLTIPS,
               output_backend="webgl", toolbar_location="above")

    # Making sliders
    slider = Slider(start=0., end=1., value=1., step=.01,
                    title="Comp P-value Filter")
    slider2 = Slider(start=0., end=1., value=1., step=.01,
                     title="Lottery P-value Filter")

    try:
        colors = viridis(len(outputMatrix.uniqueGroups))
    except ValueError:
        print("Error: Bokeh can only display 256 colors. You requested ",
              len(outputMatrix.uniqueGroups),
              " Please try chart style 2 instead.")
    for i in range(len(outputMatrix.uniqueGroups)):
        # Assigning data to each point
        dx = [outputMatrix.lotScores[i]]
        dy = [outputMatrix.compScores[i]]
        dcompP = [outputMatrix.compPValues[i]]
        dlotP = [outputMatrix.lotPValues[i]]
        dgroup = [outputMatrix.uniqueGroups[i]]
        dcolor = colors[i]

        source = ColumnDataSource(data=dict(
            x=dx,
            y=dy,
            groups=dgroup,
            compP=dcompP,
            lotP=dlotP,
            size=[15],
            color=[dcolor]
        ))
        # Callback for when the slider is changed
        callback = CustomJS(args=dict(source=source), code="""
            source.change.emit();
        """)
        slider.js_on_change('value', callback)
        slider2.js_on_change('value', callback)

        # Custom Javascript boolean filters
        js_filter = CustomJSFilter(args=dict(slider=slider, source=source),
                                   code="""
            bools = []
            for (i = 0; i < source.data.x.length; i++) {
                if (source.data.compP[i] < slider.value) {
                    bools.push(true);
                }
                else {
                    bools.push(false);
                }
            }

            return bools;
        """)
        js_filter2 = CustomJSFilter(args=dict(slider=slider2, source=source),
                                    code="""
            bools = []
            for (i = 0; i < source.data.x.length; i++) {
                if (source.data.lotP[i] < slider.value) {
                    bools.push(true);
                }
                else {
                    bools.push(false);
                }
            }

            return bools;
        """)

        # Using those filters to change what data is displayed
        view = CDSView(source=source, filters=[js_filter, js_filter2])
        p.circle(x="x", y="y", source=source, size="size",
                 legend="groups", color="color", view=view)

    p.legend.click_policy = "hide"  # Click legend entry to hide that point

    sliders = column(slider, slider2)
    layout = row(p, sliders)

    show(layout)


def bokeh_xy_cmap(outputMatrix):
    # outputArray is a 2d array composed of
    # a transposed array of unique group names, competitiveness scores,
    # comp p-values, lottery scores, and lottery p-values.
    # Scatterplot where comp p-val is shown through size
    # and lot p-val is shown through a color map
    output_file("XY_chart.html")

    TOOLS = ("hover,pan,wheel_zoom,zoom_in,zoom_out,box_zoom,reset, save, tap")

    TOOLTIPS = [
        ("group", "@groups"),
        ("Comp P-value", "@compP{0.0000}"),
        ("Lottery P-value", "@lotP{0.0000}"),
        ("(x,y)", "(@x, @y)")
    ]

    # create a new plot with a title and axis labels
    p = figure(tools=TOOLS, title="Scatterplot", x_axis_label='Lottery Score',
               y_axis_label='Competitiveness', tooltips=TOOLTIPS,
               output_backend="webgl", toolbar_location="above")

    mapper = linear_cmap(field_name='lotP',  # Color map based on lottery P-val
                         palette=list(reversed((viridis(20)))),
                         low=0.0, high=1.0)
    for i in range(len(outputMatrix.uniqueGroups)):
        dx = [outputMatrix.lotScores[i]]
        dy = [outputMatrix.compScores[i]]
        dcompP = [outputMatrix.compPValues[i]]
        dlotP = [outputMatrix.lotPValues[i]]
        dgroup = [outputMatrix.uniqueGroups[i]]

        source = ColumnDataSource(data=dict(
            x=dx,
            y=dy,
            groups=dgroup,
            compP=dcompP,
            lotP=dlotP,
            size=[size_clamp((1 - dcompP[0]) * 30)]
        ))
        p.circle(x="x", y="y", source=source, size="size",
                 legend="groups", color=mapper)

    p.legend.click_policy = "hide"

    # Bar showing what color corresponds to what
    color_bar = ColorBar(color_mapper=mapper['transform'], width=10,
                         location=(0, 0), title="Lottery P-value")

    p.add_layout(color_bar, 'left')

    show(p)


def bokeh_xy(outputMatrix):
    # outputArray is a 2d array composed of
    # a transposed array of unique group names, competitiveness scores,
    # comp p-values, lottery scores, and lottery p-values.
    # Scatterplot where lot and comp p-values are shown through size of points.
    output_file("XY_chart.html")

    TOOLS = ("hover,pan,wheel_zoom,zoom_in,zoom_out,box_zoom,reset, save, tap")

    TOOLTIPS = [
        ("group", "@groups"),
        ("Comp P-value", "@compP{0.0000}"),
        ("Lottery P-value", "@lotP{0.0000}"),
        ("(x,y)", "(@x, @y)")
    ]

    # create a new plot with a title and axis labels
    p = figure(tools=TOOLS, title="Scatterplot", x_axis_label='Lottery Score',
               y_axis_label='Competitiveness', tooltips=TOOLTIPS,
               output_backend="webgl", toolbar_location="above")

    try:
        colors = viridis(len(outputMatrix.uniqueGroups))
    except ValueError:
        print("Error: Bokeh can only display 256 colors. You requested ",
              len(outputMatrix.uniqueGroups),
              " Please try chart style 2 instead.")
    for i in range(len(outputMatrix.uniqueGroups)):
        dx = [outputMatrix.lotScores[i]]
        dy = [outputMatrix.compScores[i]]
        dcompP = [outputMatrix.compPValues[i]]
        dlotP = [outputMatrix.lotPValues[i]]
        dgroup = [outputMatrix.uniqueGroups[i]]
        dcolor = colors[i]

        source = ColumnDataSource(data=dict(
            x=dx,
            y=dy,
            groups=dgroup,
            compP=dcompP,
            lotP=dlotP,
            size=[size_clamp((1 - dcompP[0]) * (1 - dlotP[0]) * 50)],
            color=[dcolor]
        ))
        p.circle(x="x", y="y", source=source, size="size",
                 legend="groups", color="color")

    p.legend.click_policy = "hide"
    show(p)


def size_clamp(x):
    if x < 5:
        return 5
    if x > 50:
        return 50
    return x



from random import shuffle
import numpy as np

from .diversity import shannon_diversity, lottery_shannon
from .groups import group_submatrix


def null_dists_comp(data, groups, nullSize):
    # data is a 2d array of all the numerical data of species across samples
    # from the input file. groups is a list of the groups that each species
    # belongs to taken from the final column of the inpt file.
    # nullSize is how many rows of null data are in the final array.
    # Returns a list containing the null distributions of all groups
    # for comp scores.
    uniqueGroups = np.unique(groups)
    nullDists = np.empty((len(uniqueGroups), nullSize, data.shape[1]))

    for i in range(len(uniqueGroups)):
        group = uniqueGroups[i]
        nullDists[i] = null_dist_comp(data, groups, group, nullSize)
    return nullDists


def null_dist_comp(data, groups, group, nullSize):
    # data is a 2d array of all the numerical data of species across samples
    # from the input file. groups is a list of the groups that each species
    # belongs to taken from the final column of the inpt file.
    # group is an element of groups and is the specific group that a null dist
    # will be generated for. nullSize is how many rows of null data are
    # in the final array.
    # Creates a null distribution of a group by shuffling groups
    # nullSize times and calculating the shannon diversity of the
    # shuffled groups

    nullDist = np.empty((nullSize, data.shape[1]))
    shuffledGroups = groups.copy()  # Creating a copy to shuffle in place
    for i in range(nullSize):
        shuffle(shuffledGroups)
        groupSubmatrix = group_submatrix(data, shuffledGroups, group)
        nullDist[i] = np.array(shannon_diversity(groupSubmatrix))
    return nullDist


def null_dist_lottery(groupSubmatrix, nullSize):
    # groupSubmatrix is a 2d array of numerical data of species across samples
    # for a specific group. nullSize is how many rows of null data are
    # in the final array.
    # Creates a null distribution of a lottery group by shuffling each column
    # of a group nullSize times and calculating the shannon diversity of the
    # row sum

    # Creating an empty copy of groupSubmatrix (same dimensions)
    nullDist = np.empty((nullSize, groupSubmatrix.shape[1]))

    for i in range(nullSize):
        # Creating an empty array the size of the submatrix
        shuffledGroup = np.empty(groupSubmatrix.shape, dtype=int)
        for x in range(groupSubmatrix.shape[1]):
            # Splicing the xth column
            column = groupSubmatrix[0:, x].copy()
            shuffle(column)

            # Place column back in a new matrix
            shuffledGroup[:, x] = column

        shannonArray = np.array(lottery_shannon(shuffledGroup))
        nullDist[i] = shannonArray

    return nullDist


def null_dists_lottery(data, groups, nullSize):
    # data is a 2d array of all the numerical data of species across samples
    # from the input file. groups is a list of the groups that each species
    # belongs to taken from the final column of the inpt file.
    # nullSize is how many rows of null data are in the final array.
    # Returns a list containing the null distributions of all groups
    # for lot scores.

    uniqueGroups = np.unique(groups)
    nullDists = np.empty((len(uniqueGroups), nullSize, data.shape[1]))

    for i in range(len(uniqueGroups)):
        group = uniqueGroups[i]
        nullDists[i] = null_dist_lottery(group_submatrix(data, groups, group),
                                         nullSize)
    return nullDists



import numpy as np


class InputMatrix:
    # This class groups all of the related data pulled from the input file

    def __init__(self, data, groups, sampleHeaders, speciesHeaders):
        self.data = data
        self.groups = groups
        self.sampleHeaders = sampleHeaders
        self.speciesHeaders = speciesHeaders
        self.uniqueGroups = np.unique(groups)


class OutputMatrix:
    # This class groups all of the related data pulled generated as output

    def __init__(self, lotScores, compScores, lotPValues, compPValues,
                 uniqueGroups, sampleHeaders, speciesHeaders, outputArray):
        self.lotScores = lotScores
        self.compScores = compScores
        self.lotPValues = lotPValues
        self.compPValues = compPValues
        self.uniqueGroups = uniqueGroups
        self.sampleHeaders = sampleHeaders
        self.speciesHeaders = speciesHeaders
        self.outputArray = outputArray



import numpy
import statsmodels.stats.multitest as stm

from .diversity import shannon_diversity_indv


def p_values(null, shDiv, twoTailedFlag, alpha):
    # Returns an fdr corrected p-value that is one or two tailed.
    # Null is a 2d array of null values for a group
    # generated through monte carlo simulation,
    # shDiv is a list of empirical shannon diversity values for a group,
    # TwoTailedFlag specifies whether it should be one or two-tailed,
    # Alpha is the threshold value we compare the p-values against.
    if twoTailedFlag:
        return stm.multipletests(two_tailed_comparison(null, shDiv), alpha,
                                 'hs')[1]
    return stm.multipletests(one_tailed_lower(null, shDiv), alpha,
                             'hs')[1]


def two_tailed_comparison(null, shDiv):  # Two-tailed p-value
    # Takes a 2d array of null values and a list of the empirical shannon
    # diversities for a group. Returns the upper or lower one tailed p-value
    # based on which is more significant.
    u = numpy.median(shDiv)
    mid = numpy.median(null)
    if u > mid:
        p = one_tailed_upper(null, shDiv)
    else:
        p = one_tailed_lower(null, shDiv)

    return p


def one_tailed_lower(null, shDiv):  # One-tailed lower p-value
    # Takes a 2d array of null values and a list of the empirical shannon
    # diversities for a group. Returns the p-value given by the proportion
    # of values greater than u in null.
    u = numpy.median(shDiv)
    n_below = numpy.sum(numpy.greater(u, null))
    if n_below <= 0:
        return 1/null.size
    else:
        return n_below / null.size


def one_tailed_upper(null, shDiv):  # One-tailed upper p-value
    # Takes a 2d array of null values and a list of the empirical shannon
    # diversities for a group. Returns the p-value given by the proportion
    # of values less than u in null.
    u = numpy.median(shDiv)
    n_below = numpy.sum(numpy.less(u, null))
    if n_below <= 0:
        return 1/null.size
    else:
        return n_below / null.size


def standard_effect(null, shDiv):  # Competitiveness score
    # Takes a 2d array of null values and a list of the empirical shannon
    # diversities for a group.
    # Returns the standard effect size for a group based off the formula below:
    # https://en.wikipedia.org/wiki/Effect_size#Difference_family:_Effect_sizes_based_on_differences_between_means
    return ((numpy.mean(shDiv) - numpy.mean(null))/numpy.std(null))



from src import csv_IO
from numpy import array
from numpy import array_equal


def test_csv_reader_h():
    data = array([[20, 50, 10, 20, 0], [30, 30, 30, 10, 0],
                  [10, 15, 20, 45, 10], [5, 5, 5, 0, 85],
                  [15, 0, 15, 10, 0], [20, 0, 20, 15, 5]])
    groups = array(["C", "C", "C", "H", "H", "H"])
    sampleHeaders = array(["", "A", "B", "C", "D", "E", ""], dtype=str)
    speciesHeaders = array(["1", "2", "3", "4", "5", "6"], dtype=str)
    inputMatrix = csv_IO.csv_reader("data/testcsv.csv", False)

    assert array_equal(data, inputMatrix.data)
    assert array_equal(groups, inputMatrix.groups)
    assert array_equal(speciesHeaders, inputMatrix.speciesHeaders)
    assert array_equal(sampleHeaders, inputMatrix.sampleHeaders)


def test_csv_reader_nh():
    data = array([[20, 50, 10, 20, 0], [30, 30, 30, 10, 0],
                  [10, 15, 20, 45, 10], [5, 5, 5, 0, 85],
                  [15, 0, 15, 10, 0], [20, 0, 20, 15, 5]])
    groups = array(["C", "C", "C", "H", "H", "H"])
    sampleHeaders = array(["", "", "", "", "", ""], dtype=str)
    speciesHeaders = array(["", "", "", "", "", ""], dtype=str)
    inputMatrix = csv_IO.csv_reader("data/testcsvNH.csv", True)

    assert array_equal(data, inputMatrix.data)
    assert array_equal(groups, inputMatrix.groups)
    assert array_equal(speciesHeaders, inputMatrix.speciesHeaders)
    assert array_equal(sampleHeaders, inputMatrix.sampleHeaders)



import numpy as np
import sys

from .dataMatrix import InputMatrix


def csv_reader(filename, noHeaderFlag):
    # Reads csv data to numpy arrays and outputs a InputMatrix containing them.
    # filename is the name of the file that will be read from i.e 'input.csv'
    # noHeaderFlag specifies that the input file doesn't have headers if true.

    try:
        with open(filename) as csvfile:
            # I would love to implement a lazy loading iterator approach here
            # to avoid effectively duplicating the input array during splicing,
            # but not sure on implementation. Maybe for later.
            inputArray = np.genfromtxt(csvfile, delimiter=',', dtype=str)

            if not noHeaderFlag:  # If headers exist
                sampleHeaders = np.reshape(inputArray[:1], inputArray.shape[1])
                speciesHeaders = inputArray[1:, 0]
                groups = inputArray[1:, -1]
                data = inputArray[1:, 1:-1].astype(int)
            else:
                speciesHeaders = np.full(inputArray.shape[1], "")
                sampleHeaders = np.full(inputArray.shape[0], "")
                groups = inputArray[0:, -1]
                data = inputArray[0:, 0:-1].astype(int)

            return InputMatrix(data, groups, sampleHeaders, speciesHeaders)

    except FileNotFoundError:
        print("File not found at", filename, file=sys.stderr)
        sys.exit()


def csv_writer(filename, outputArray):
    # Outputs data to a csv file.
    # filename is the name of the file that will be written to.
    # outputArray is the array that will be written to file composed of
    # a transposed array of unique group names, competitiveness scores,
    # comp p-values, lottery scores, and lottery p-values.

    try:
        np.savetxt(filename, outputArray,
                   delimiter=",", fmt="%s,%.15s,%.15s,%.15s,%.15s",
                   header=("Group,Comp Score,Comp Pval,"
                           "Lot Score,Lot Pval"),
                   comments="")
    except PermissionError:
        print("File output failed. Permissions not granted for", filename,
              file=sys.stderr)
    except:
        print("File output failed. Unknown Error.", file=sys.stderr)



from src import groups
from numpy import array
from numpy import array_equal
from numpy import unique


def test_group_submatrix():
    # Testing if the array generated by group_submatrix
    # matches the expected values
    data = array([[20, 50, 10, 20, 0], [30, 30, 30, 10, 0],
                  [10, 15, 20, 45, 10], [5, 5, 5, 0, 85],
                  [15, 0, 15, 10, 0], [20, 0, 20, 15, 5]])
    groupArray = array(["C", "C", "C", "H", "H", "H"], dtype=str)
    groupName = "C"
    submatrix = array([[20, 50, 10, 20, 0], [30, 30, 30, 10, 0],
                       [10, 15, 20, 45, 10]])

    assert array_equal(groups.group_submatrix(data, groupArray,
                                              groupName), submatrix), "Generated submatrix doesn't match expected values."


def test_group_submatrices():
    # Testing if the array of group submatrices matches the expected values
    data = array([[20, 50, 10, 20, 0], [30, 30, 30, 10, 0],
                  [10, 15, 20, 45, 10], [5, 5, 5, 0, 85],
                  [15, 0, 15, 10, 0], [20, 0, 20, 15, 5]])
    groupArray = array(["C", "C", "C", "H", "H", "H"], dtype=str)
    uniqueGroups = unique(groupArray)

    submatrices = array([[[20, 50, 10, 20, 0], [30, 30, 30, 10, 0],
                          [10, 15, 20, 45, 10]],
                         [[5, 5, 5, 0, 85], [15, 0, 15, 10, 0],
                          [20, 0, 20, 15, 5]]])

    assert array_equal(groups.group_submatrices(data, groupArray,
                                                uniqueGroups), submatrices), "Generated submatrices doesn't match expected values."



import setuptools


with open("README.txt", "r") as fh:
    long = fh.read()

setuptools.setup(
    # Project information
    name="powerball",
    version="0.1.0",
    author="Matthew Marshall",
    author_email="matthewmarshall124@gmail.com",
    url="https://github.com/mmarshall124/powerball",
    license="MIT",

    # Description
    description="A tool for competitive lottery analysis of bacterial groups.",
    long_description=long,
    long_description_content_type="text/plain",
    # Packages
    packages=setuptools.find_packages(),
    # Classifiers
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Development Status :: 3 - Alpha",
        "Environment :: Console",
        "Topic :: Scientific/Engineering :: Bio-Informatics",
    ],



)


from numpy import array
import numpy


def make_3d_jagged_array(nrows, ncol):
    # nrows: array of ints, specifying the number
    # of rows to output in each subarray.
    # wrapper function that takes only one arg,
    # needed for map. y is number of cols for
    # all subarrays.
    def empty_wrapper(nrows, y=ncol): return(numpy.empty((nrows, y)))
    return list(map(empty_wrapper, nrows))


def group_submatrices(data, groups, unique):
    # data - a 2d array (D below)
    # gets colums of matrix "data" subset by each unique element within groups
    # unique - is a list of unique groups from groups.
    # This could be used to filter out specific groups,
    # say ones that don't meet the minimum group size.
    # However, for now it is equivalent to numpy.unique(groups)
    # Returns an array of the submatrix of each group
    groupSizes = [0] * len(unique)
    for i in range(len(unique)):
        groupSizes[i] = sum(groups == unique[i])

    submatrices = make_3d_jagged_array(nrows=groupSizes, ncol=len(data[0]))

    for i in range(len(unique)):
        submatrices[i] = group_submatrix(data, groups, unique[i])

    return array(submatrices)


def group_submatrix(data, groups, groupName):
    # # data is a 2d array of all the numerical data of species across samples
    # from the input file. groups is a list of the groups that each species
    # belongs to taken from the final column of the inpt file.
    # group is an element of groups and is the specific group that a submatrix
    # will be generated for.
    # Returns a 2d array of the data for a single group specified by group
    rows = (groups == groupName)
    cols = [True] * len(data[0])
    return(data[numpy.ix_(rows, cols)])
"""
def group_submatrices(data, groups, unique):
    # This function returns an array of the submatrix of each group

    Submatrices = []  # This is where we'll store the end results

    for groupName in unique:
        submatrix = group_submatrix(data, groups, groupName)
        Submatrices.append(submatrix)

    return array(Submatrices)
def group_submatrix(data, groups, groupName):
    # Returns the data for a single group specified by groupName

    submatrix = []
    # Get the submatrix where all members belong to the same group
    for i in range(data.shape[0]):
        if groupName == groups[i]:  # If the group name == that row's group
            submatrix.append(data[i])
    return array(submatrix)
"""



from math import log


def shannon_diversity(group_submatrix):
    # groupSubmatrix is a 2d array of numerical data of species across samples
    # for a specific group.
    # This function calculates the shannon_diversity for a group submatrix

    # Break the submatrix into a list of its columns
    column_list = [column(group_submatrix, i)
                   for i in range(len(group_submatrix[0]))]

    # Return a list of each columns shannon diversity for the group submatrix
    return [shannon_diversity_indv(i) for i in column_list]


def shannon_diversity_indv(group_column):
    # Returns the shannon diversity for an individual column in a group
    # H = -SUM[(pi) * ln(pi)] where pi = Number species/Number of samples
    col_sum = sum(group_column)
    if col_sum == 0:
        return 0

    pi_x_logs = []
    for y in group_column:
        pi = y/col_sum
        if pi == 0:  # log(0) = -infinity
            continue
        pi_x_logs.append(pi * log(pi))
    return -sum(pi_x_logs)


def lottery_shannon(groupSubmatrix):
    # groupSubmatrix is a 2d array of numerical data of species across samples
    # for a specific group.
    # Calculates the shannon diversity of row sums used to generate lot scores
    # Sum across rows, store in list
    summedRows = []
    for row in groupSubmatrix:
        summedRows.append(sum(row))
    # Return the Shannon diversity
    return shannon_diversity_indv(summedRows)


def column(matrix, i):
    # Returns a list made of the ith column of the matrix
    return [row[i] for row in matrix]



from src import statFunctions
from numpy import array


def test_two_tailed_upper():
    # Test the two tailed p-value function on the upper end on
    # a generated list from 1-100
    testList = array(range(1, 101))
    u = 60
    assert statFunctions.two_tailed_comparison(testList, u) == 0.4


def test_two_tailed_lower():
    # Test the two tailed p-value function on the lower end on
    # a generated list from 1-100
    testList = array(range(1, 101))
    u = 40
    assert statFunctions.two_tailed_comparison(testList, u) == 0.39


def test_two_tailed_zero():
    # Testing the two tailed comparison test on zero on
    # a generated list from 1-100
    testList = array(range(1, 101))
    u = 100
    assert statFunctions.two_tailed_comparison(testList, u) == 0.01


def test_two_tailed_mid():
    # Testing the two tailed comparison test on the middle value on
    # a generated list from 1-100
    testList = array(range(1, 101))
    u = 50
    assert statFunctions.two_tailed_comparison(testList, u) == 0.49


def test_one_tailed_low():
    # Testing the one tailed lower comparison tests on
    # a generated list from 1-100
    testList = array(range(1, 101))
    u = 40
    assert statFunctions.one_tailed_lower(testList, u) == 0.39
    assert statFunctions.one_tailed_upper(testList, u) == 0.6


def test_one_tailed_high():
    # Testing the two tailed comparison test on zero on
    # a generated list from 1-100
    testList = array(range(1, 101))
    u = 60
    assert statFunctions.one_tailed_lower(testList, u) == 0.59
    assert statFunctions.one_tailed_upper(testList, u) == 0.4


def test_one_tailed_zero():
    # Testing the two tailed comparison test on zero on
    # a generated list from 1-100
    testList = array(range(1, 101))
    u = 0
    assert statFunctions.one_tailed_lower(testList, u) == 0.01
    assert statFunctions.one_tailed_upper(testList, u) == 1


def test_one_tailed_mid():
    # Testing the two tailed comparison test on zero on
    # a generated list from 1-100
    testList = array(range(1, 101))
    u = 50
    assert statFunctions.one_tailed_lower(testList, u) == 0.49
    assert statFunctions.one_tailed_upper(testList, u) == 0.5
